//LTC2310 Verilog Driver for Altera Cyclone III FPGA
//Data format from ADC is CMOS externally clocked
//Input clock rate (42MHz) is 21 times the output clock rate for the 12/14/16 bit parts
//Note that D[] bits are read on ch 1 of PSCOPE
//The inputs BITSEL1, BITSEL2 select between 12, 14, & 16 bit formats as follows:
//BITSEL1 = 0, BITSEL2 = X: select 12 bits
//BITSEL1 = 1, BITSEL2 = 0: select 14 bits
//BITSEL1 = 1, BITSEL2 = 1: select 16 bits
// The timing constraints for this device are:
// CNV pulse width >25nsec
// Time from CNV falling edge to first SCK falling edge >9.5nsec

//Uses negedge 45 deg delayed 'rx_clk_42' to clock in data
//Uses an 'rx_start' pulse during the first negedge to enable clocking the MSB

//DJS 5_17_16

`timescale 1ps / 1ps
module LTC2310_cmos
(	
	CLKIN,
	SCK,
	CNVCLK,
	CSB,
	SDO,
	D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15,
	BITSEL1,BITSEL2
);

input BITSEL1, BITSEL2;
input CLKIN;			//Clock in from user @ 42/42/42 MHz
input	SDO;				//Data out from ADC

output CSB;				//Convert Start on falling edge							
output SCK;				//SCK from FPGA to ADC; data appears on 'SDO' after each falling edge of 'SCK'
output CNVCLK;			//Latch signal from FPGA to DC890 on rising edge
output D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15;

reg SCK;	
reg [15:0] D;
reg [5:0] num_bits;

wire CLKIN;
wire CSB;																																		
wire CNVCLK;														
wire sck_gate,clk_42,clk_84,sync,rx_clk_42,rx_start;
wire D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15;						
wire SDO;
wire [15:0] data;
  
always @ (CLKIN)  //decode the number of bits
begin
  if(BITSEL1 == 1)
    begin
      if(BITSEL2 == 1)  //16 bits
			num_bits = 5'd16;
      else  //14 bits
			num_bits = 5'd16;	//note the code has to clock out 16 bits for the 14 bit part
	 end
  else  //12 bits
	num_bits = 5'd14;	//note the code has to clock out 14 bits for the 12 bit part
end
  
sync_pll	synchronize(
	.inclk0(CLKIN),
	.c0(clk_42),
	.c1(clk_84),
	.c2(sync),
	.c3(rx_clk_42)
);

//generate the 'SCK' signal
always @(clk_42)
	SCK = ~(clk_42 && sck_gate);

//generate the timing pulses
pulse_generator pulses
(
	.clk(clk_84),
	.sync(sync),
	.csb(CSB),
	.cnvclk(CNVCLK),
	.sck_gate(sck_gate),
	.rx_start(rx_start),
	.num_bits(num_bits)
);

// Receive serial data from ADC and converts it to parallel format
deserializer rx
(
	.rx_start(rx_start),
	.clk(rx_clk_42),
	.sdo(SDO),						  
	.parallel_data(data),
	.num_bits(num_bits)
);

assign D0 = D[0];
assign D1 = D[1];
assign D2 = D[2];
assign D3 = D[3];
assign D4 = D[4];
assign D5 = D[5];
assign D6 = D[6];
assign D7 = D[7];
assign D8 = D[8];
assign D9 = D[9];
assign D10 = D[10];
assign D11 = D[11];
assign D12 = D[12];
assign D13 = D[13];
assign D14 = D[14];
assign D15 = D[15];

//load the parallel data from the channel onto the 16 bit bus
always @ (posedge CNVCLK)
	 D = data;

endmodule
